Re: [RFC/PATCH 4/5] completion: get rid of compgen
- From
Felipe Contreras <felipe.contreras@gmail.com>
- Date
- Nov 17, 2012, 19:33 UTC
- Message-ID
- <CAMP44s0YnoGwJEgUbXZ831_OrgO=dDf5_QHxT5JYnGUHYPuiTw@mail.gmail.com>
- In-Reply-To
- <20121117141215.GG12052@goldbirke>
On Sat, Nov 17, 2012 at 3:12 PM, SZEDER Gábor <szeder@ira.uka.de> wrote:
Show 59 quoted lines
> On Sat, Nov 17, 2012 at 12:42:38PM +0100, Felipe Contreras wrote:
>> On Sat, Nov 17, 2012 at 12:00 PM, SZEDER Gábor <szeder@ira.uka.de> wrote:
>> > On Sat, Nov 17, 2012 at 02:38:17AM +0100, Felipe Contreras wrote:
>> >> The functionality we use is very simple, plus, this fixes a known
>> >> breakage 'complete tree filename with metacharacters'.
>> >>
>> >> Signed-off-by: Felipe Contreras <felipe.contreras@gmail.com>
>> >> ---
>> >> contrib/completion/git-completion.bash | 6 +++++-
>> >> 1 file changed, 5 insertions(+), 1 deletion(-)
>> >>
>> >> diff --git a/contrib/completion/git-completion.bash b/contrib/completion/git-completion.bash
>> >> index 975ae13..ad3e1fe 100644
>> >> --- a/contrib/completion/git-completion.bash
>> >> +++ b/contrib/completion/git-completion.bash
>> >> @@ -227,7 +227,11 @@ fi
>> >>
>> >> __gitcompadd ()
>> >> {
>> >> - COMPREPLY=($(compgen -W "$1" -P "$2" -S "$4" -- "$3"))
>> >> + for x in $1; do
>> >> + if [[ "$x" = "$3"* ]]; then
>> >> + COMPREPLY+=("$2$x$4")
>> >> + fi
>> >> + done
>> >
>> > The whole point of creating __gitcomp_nl() back then was to fill
>> > COMPREPLY without iterating through all words in the wordlist, making
>> > completion faster for large number of words, e.g. a lot of refs, or
>> > later a lot of symbols for 'git grep' in a larger project.
>> >
>> > The loop here kills that optimization.
>>
>> So your solution is to move the loop to awk? I fail to see how that
>> could bring more optimization, specially since it includes an extra
>> fork now.
>
> This patch didn't aim for more optimization, but it was definitely a
> goal not to waste what we gained by creating __gitcomp_nl() in
> a31e6262 (completion: optimize refs completion, 2011-10-15). However,
> as it turns out the new version with awk is actually faster than
> current master with compgen:
>
> Before:
>
> $ refs="$(for i in {0..9999} ; do echo branch$i ; done)"
> $ time __gitcomp_nl "$refs"
>
> real 0m0.242s
> user 0m0.220s
> sys 0m0.028s
>
> After:
>
> $ time __gitcomp_nl "$refs"
>
> real 0m0.109s
> user 0m0.096s
> sys 0m0.012sThis one is even faster:
while read -r x; do
if [[ "$x" == "$3"* ]]; then
COMPREPLY+=("$2$x$4")
fi
done <<< $1== 10000 == one: real 0m0.148s user 0m0.134s sys 0m0.025s two: real 0m0.055s user 0m0.054s sys 0m0.000s
-- Felipe Contreras